package cn.edu.whu.boot.statistics.bean;

public class UserWorkload {
    private String realName;

    // 巡查人员分配到的任务量
    private Long reviewerTotalWorkload;

    // 该用户采集的数据量
    private Long collectedDataCount;

    // 该用户采集的数据中，待审核的数据量
    private Long toBeReviewedDataCount;

    // 该用户采集的数据中，通过审核的数据量
    private Long validDataCount;

    // 该用户采集的数据中，未通过审核的数据量
    private Long invalidDataCount;

    // 该用户额外新采集的数据量（不包括从网页端导入的数据）
    private Long newDataCount;

    public UserWorkload() {
        reviewerTotalWorkload = 0L;
        collectedDataCount = 0L;
        toBeReviewedDataCount = 0L;
        validDataCount = 0L;
        invalidDataCount = 0L;
        newDataCount = 0L;
    }

    public UserWorkload(String realName) {
        this();
        this.realName = realName;
    }

    public String getRealName() {
        return realName;
    }

    public void setRealName(String realName) {
        this.realName = realName;
    }

    public Long getReviewerTotalWorkload() {
        return reviewerTotalWorkload;
    }

    public void setReviewerTotalWorkload(Long reviewerTotalWorkload) {
        this.reviewerTotalWorkload = reviewerTotalWorkload;
    }

    public void incrementReviewerTotalWorkload(long reviewerTotalWorkload) {
        this.reviewerTotalWorkload += reviewerTotalWorkload;
    }

    public Long getCollectedDataCount() {
        return collectedDataCount;
    }

    public void setCollectedDataCount(Long collectedDataCount) {
        this.collectedDataCount = collectedDataCount;
    }

    public void incrementCollectedDataCount(long collectedDataCount) {
        this.collectedDataCount += collectedDataCount;
    }

    public Long getToBeReviewedDataCount() {
        return toBeReviewedDataCount;
    }

    public void setToBeReviewedDataCount(Long toBeReviewedDataCount) {
        this.toBeReviewedDataCount = toBeReviewedDataCount;
    }

    public void incrementToBeReviewedDataCount(long toBeReviewedDataCount) {
        this.toBeReviewedDataCount += toBeReviewedDataCount;
    }

    public void incrementReviewerDoneWorkload(long reviewerDoneWorkload) {
    }

    public Long getValidDataCount() {
        return validDataCount;
    }

    public void setValidDataCount(Long validDataCount) {
        this.validDataCount = validDataCount;
    }

    public void incrementValidDataCount(long validDataCount) {
        this.validDataCount += validDataCount;
    }

    public Long getInvalidDataCount() {
        return invalidDataCount;
    }

    public void setInvalidDataCount(Long invalidDataCount) {
        this.invalidDataCount = invalidDataCount;
    }

    public void incrementInvalidDataCount(long invalidDataCount) {
        this.invalidDataCount += invalidDataCount;
    }

    public Long getNewDataCount() {
        return newDataCount;
    }

    public void setNewDataCount(Long newDataCount) {
        this.newDataCount = newDataCount;
    }

    public void incrementNewDataCount(long newDataCount) {
        this.newDataCount += newDataCount;
    }
}
